import React, { Component } from 'react';
import PropTypes from 'prop-types';
import DayPicker, { DateUtils } from 'react-day-picker';
import dateFns from 'date-fns';
import WithClickOutside from '../with-click-out-side';
import StyledDateRangePicker from './DateRangePicker.style';

class DateRangePicker extends Component {
    constructor(props) {
        super(props);

        this.state = {
            showMonth: props.from || new Date(),
            hoveredToDate: props.to,
            showDateTimePicker: false,
        };
    }

    getCalendarHeader = (showMonth, numberOfMonthsAhead) => {
        const options = [];
        for (let i = 0; i < numberOfMonthsAhead + 1; i++) {
            const month = dateFns.addMonths(new Date(), i);
            const monthStr = dateFns.format(month, 'MMMM YYYY');
            const monthVal = dateFns.format(month, 'YYYY-MM-01');
            options.push({ value: monthVal, label: monthStr });
        }

        const label = dateFns.format(showMonth, 'MMMM YYYY');
        return (
            <div className="header">
                <h2>
                    {label}
                </h2>
            </div>
        );
    }

    handleDayClick = (day, modifiers = {}) => {
        if (modifiers.disabled) return;

        const { from, to, onDayChanged } = this.props;
        let range = { from, to };
        // When both from & to have values, then reset range and hoveredToDate
        if (from && to) {
            range = { from: undefined, to: undefined };
            this.setState({ hoveredToDate: null });
        }

        const newRange = day instanceof Date ? DateUtils.addDayToRange(day, range) : range;
        if (newRange.from && newRange.to) this.closeDatePicker();

        if (onDayChanged) onDayChanged(newRange);
    }

    clearSelection = () => {
        const { onDayChanged } = this.props;
        if (onDayChanged) onDayChanged({ from: undefined, to: undefined });
    }

    closeDatePicker = () => {
        this.setState({ showDateTimePicker: false });
    }

    openDatePicker = () => {
        this.setState({ showDateTimePicker: true });
    }

    handleDayMouseEnter = (day, modifiers = {}) => {
        if (modifiers.disabled) return;

        const { from, to } = this.props;
        if ((from && !to) || (!from && to)) this.setState({ hoveredToDate: day });
    }

    handleMonthSelected = (selectedOption) => {
        const month = new Date(
            Date.parse(dateFns.format(selectedOption.value, 'YYYY-MM-01'))
        );
        this.setState({ showMonth: month });
    }

    handleMonthChange = (month) => {
        this.setState({ showMonth: month });
    }

    render() {
        const { from, to, maxAvailableMonths } = this.props;
        const { showMonth, hoveredToDate, showDateTimePicker } = this.state;
        const today = new Date();
        const selectedDate = from && to ? `${dateFns.format(from, 'D/M/YYYY')} - ${dateFns.format(to, 'D/M/YYYY')}` : '';

        return (
            <StyledDateRangePicker>
                <span className="date-picker form-input icon-textfield" onClick={this.openDatePicker}>
                    <input type="text" className="standard clearfield-option" placeholder="Select date range" value={selectedDate} />
                    <span className="wj-icon wj-calendar"></span>
                    {
                        selectedDate && <span className="wj-icon wj-alt-cross" onClick={this.handleDayClick}></span>
                    }
                </span>
                <WithClickOutside onClickOutside={this.closeDatePicker}>
                    {
                        setWrapperRef => (
                            <div ref={setWrapperRef}>
                                {
                                    showDateTimePicker && (
                                        <div className="wj-rc-datepicker">
                                            {this.getCalendarHeader(showMonth, maxAvailableMonths)}
                                            <DayPicker
                                                selectedDays={{ from, to: hoveredToDate }}
                                                month={showMonth}
                                                showOutsideDays
                                                modifiers={{ start: from, end: hoveredToDate }}
                                                onDayClick={this.handleDayClick}
                                                onMonthChange={this.handleMonthChange}
                                                onDayMouseEnter={this.handleDayMouseEnter}
                                                weekdaysShort={['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat']}
                                            />
                                        </div>
                                    )
                                }
                            </div>
                        )
                    }
                </WithClickOutside>
            </StyledDateRangePicker>
        );
    }
}

DateRangePicker.propTypes = {
    from: PropTypes.objectOf(Date),
    to: PropTypes.objectOf(Date),
    maxAvailableMonths: PropTypes.number,
    onDayChanged: PropTypes.func.isRequired,
};

DateRangePicker.defaultProps = {
    from: undefined,
    to: undefined,
    maxAvailableMonths: 11,
};

export default DateRangePicker;
